@@ -1,5 +1,5 @@
 /*
-$Id: dilib.c,v 1.31 2006-03-29 12:50:42-08 bll Exp $
+$Id: dilib.c,v 1.34 2006-10-04 23:51:52-07 bll Exp $
 $Source: /export/home/bll/DI/RCS/dilib.c,v $
 Copyright 1994-2006 Brad Lanam, Walnut Creek, CA
 */
@@ -312,7 +312,7 @@ di_getDiskEntries (diskInfo, diCount)
     BEntry          entry;
     BPath           path;
 
-    if (di_lib_debug > 0) { printf ("# lib:getDiskEntries: _lib_fs_stat_dev\n"); }
+    if (di_lib_debug > 0) { printf ("# lib:getDiskEntries: fs_stat_dev\n"); }
     count = 0;
     while ((dev = next_dev (&count)) != B_BAD_VALUE)
     {
@@ -557,6 +557,11 @@ di_getDiskInfo (diskInfo, diCount)
                 {
                     diptr->blockSize = statBuf.f_frsize;
                 }
+/* Linux! statvfs() returns values in f_bsize rather f_frsize.  Bleah.  */
+/* Non-POSIX!  Linux manual pages are incorrect.                        */
+#  if linux
+                diptr->blockSize = statBuf.f_bsize;
+#  endif /* linux */
 
                 diptr->totalBlocks = (_fs_size_t) statBuf.f_blocks;
                 diptr->freeBlocks = (_fs_size_t) statBuf.f_bfree;
@@ -1138,7 +1143,7 @@ di_getDiskEntries (diskInfo, diCount)
             diptr->isReadOnly = TRUE;
         }
 # endif
-        convertMountOptions (mntbufp [idx].f_flags, diptr);
+        convertMountOptions ((long) mntbufp [idx].f_flags, diptr);
         trimChar (diptr->options, ',');
 
         if (di_lib_debug > 1)
@@ -1234,7 +1239,7 @@ di_getDiskEntries (diskInfo, diCount)
         }
 # endif
         sp = mntbufp + idx;
-        convertMountOptions (sp->f_flags, diptr);
+        convertMountOptions ((long) sp->f_flags, diptr);
 # if _mem_f_type_statfs
 #  if defined (MOUNT_NFS3)
         if (sp->f_type == MOUNT_NFS3)
@@ -1365,7 +1370,7 @@ di_getDiskEntries (diskInfo, diCount)
             diptr->isLocal = TRUE;
         }
 # endif
-        convertMountOptions (sp->f_flag, diptr);
+        convertMountOptions ((long) sp->f_flag, diptr);
         trimChar (diptr->options, ',');
 
         if (sp->f_frsize == 0 && sp->f_bsize != 0)
@@ -1527,7 +1532,7 @@ di_getDiskEntries (diskInfo, diCount)
         {
             diptr->isReadOnly = FALSE;
         }
-        convertMountOptions (fsdbuf [idx].fd_req.flags, diptr);
+        convertMountOptions ((long) fsdbuf [idx].fd_req.flags, diptr);
         trimChar (diptr->options, ',');
 
         if (di_lib_debug > 1)
@@ -1702,7 +1707,6 @@ di_getDiskEntries (diskInfo, diCount)
 
         strncpy (diptr->options, (char *) vmt2dataptr (vmtp, VMT_ARGS),
                  DI_OPT_LEN);
-        /* convertMountOptions (vmtp->vmt_flags, diptr); */
         trimChar (diptr->options, ',');
 
         strncpy (diptr->mountTime, ctime ((time_t *) &vmtp->vmt_time),
