@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# $Id: cflags.sh,v 1.23 2005-11-20 19:04:34-08 bll Exp $
+# $Id: cflags.sh,v 1.30 2006-10-05 15:15:26-07 bll Exp $
 # $Source: /export/home/bll/DI/features.base/RCS/cflags.sh,v $
 # Copyright 2001-2002 Brad Lanam, Walnut Creek, California USA
 #
@@ -23,6 +23,14 @@ do
     esac
 done
 
+CC=${CC:-cc}
+ccflags=""
+ldflags=""
+libs=""
+includes=""
+usinggcc="N"
+path=$PATH
+
 ## need path separator...
 tpath=`echo $PATH | sed 's/:/ /g'`
 set ${tpath}
@@ -36,12 +44,28 @@ do
     then
         xgetconf="$i/getconf"
     fi
-    if [ -x "$i/msgfmt" ]
+    # first located
+    if [ -x "$i/msgfmt" -a "$xmsgfmt" = "" ]
     then
         xmsgfmt="$i/msgfmt"
     fi
+    # first located
+    if [ -x "$i/gmsgfmt" -a "$xgmsgfmt" = "" ]
+    then
+        xgmsgfmt="$i/gmsgfmt"
+    fi
+    if [ -x "$i/${CC}" ]
+    then
+        xccpath="$i"
+    fi
 done
 
+case ${CC} in
+    /*)
+        xccpath=`echo ${CC} | sed 's,.*/,,'`
+        ;;
+esac
+
 if [ "${xuname}" != "" ]
 then
     SYSTYPE=`${xuname} -s`
@@ -61,13 +85,6 @@ then
     esac
 fi
 
-CC=${CC:-cc}
-ccflags=""
-ldflags=""
-libs=""
-includes=""
-path=$PATH
-
 if [ "$DI_IN_BUILD" != "1" ]
 then
     ccflags="${CFLAGS:--O}"
@@ -86,9 +103,11 @@ then
     tlibs="`${xgetconf} LFS_LIBS 2>/dev/null`"
 fi
 
+# -Wtraditional
 case ${CC} in
     *gcc*)
-        ccflags="-Wall -Waggregate-return -Wconversion -Wformat -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wpointer-arith -Wshadow -Wstrict-prototypes -Wtraditional -Wunused $ccflags"
+        usinggcc="Y"
+        ccflags="-Wall -Waggregate-return -Wconversion -Wformat -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wpointer-arith -Wshadow -Wstrict-prototypes -Wunused $ccflags"
         vers=""  # we know it's gcc...don't run again
         case "${bit64}" in
             1)
@@ -98,9 +117,11 @@ case ${CC} in
         ;;
 esac
 
+# -Wtraditional
 case ${vers} in     # gcc in disguise...
     *gcc*)
-        ccflags="-Wall -Waggregate-return -Wconversion -Wformat -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wpointer-arith -Wshadow -Wstrict-prototypes -Wtraditional -Wunused $ccflags"
+        usinggcc="Y"
+        ccflags="-Wall -Waggregate-return -Wconversion -Wformat -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wpointer-arith -Wshadow -Wstrict-prototypes -Wunused $ccflags"
         case "${bit64}" in
             1)
                 ccflags="-m64 $ccflags"
@@ -111,6 +132,7 @@ esac
 
 case ${SYSTYPE} in
     AIX)
+        usinggcc="N"
         ccflags="-qhalt=e $ccflags"
         ccflags="$ccflags -qmaxmem=-1"
         case ${SYSREV} in
@@ -166,6 +188,7 @@ case ${SYSTYPE} in
                         ;;
                 esac
                 ccflags="-Ae $ccflags"
+                usinggcc="N"
                 ;;
         esac
         if [ $DI_BUILD_NO_NLS -eq 0 -a \
@@ -233,7 +256,6 @@ case ${SYSTYPE} in
 
                         case "${bit64}" in
                             1)
-                                ccflags="-xarch=v9 $ccflags"
                                 ;;
                         esac
                         ;;
@@ -250,16 +272,32 @@ ccflags="$ccflags $tccflags"
 ldflags="$ldflags $tldflags"
 libs="$libs $tlibs"
 
+mfmt="${xmsgfmt}"
+if [ "$usinggcc" = "Y" ]
+then
+    mfmt="${xgmsgfmt:-${xmsgfmt}}"
+    if [ -f "${xccpath}/msgfmt" ]
+    then
+        mfmt="${xccpath}/msgfmt"
+    fi
+    if [ -f "${xccpath}/gmsgfmt" ]
+    then
+        mfmt="${xccpath}/gmsgfmt"
+    fi
+fi
+
 echo "CC=\"${CC}\""
 echo "CFLAGS=\"$ccflags $includes\""
 echo "CINCLUDES=\"$includes\""
 echo "LDFLAGS=\"$ldflags\""
 echo "LIBS=\"$libs\""
 echo "PATH=\"$path\""
+echo "XMSGFMT=\"$mfmt\""
 echo "export CC"
 echo "export CFLAGS"
 echo "export CINCLUDES"
 echo "export LDFLAGS"
 echo "export LIBS"
 echo "export PATH"
+echo "export XMSGFMT"
 exit 0
