@@ -1,14 +1,14 @@
 /*
-$Id: hasnls.c,v 1.1 2005-07-25 10:34:24-07 bll Exp $
+$Id: hasnls.c,v 1.3 2006-09-15 13:50:19-07 bll Exp $
 $Source: /export/home/bll/DI/RCS/hasnls.c,v $
-Copyright 2005 Brad Lanam, Walnut Creek, CA
+Copyright 2006 Brad Lanam, Walnut Creek, CA
 */
 
 #include "config.h"
 #include <stdio.h>
-#include <stdlib.h>
-
-extern int di_lib_debug;
+#if _hdr_stdlib
+# include <stdlib.h>
+#endif
 
 int
 #if _proto_stdc
